#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
 *
 * TurtleBot3RosTfPub.py
 *
 *  Created on: 2022/05/31
 *      Author: T.Anazawa
 *
 *  Outline: Publish robot's local path topic.
 *
"""
import os
from datetime import datetime, timedelta
from time import strftime
import pytz
import csv
import rospy
from nav_msgs.msg import Odometry
import rosparam
import tf
from geometry_msgs.msg import TransformStamped
from geometry_msgs.msg._PoseStamped import PoseStamped
from geometry_msgs.msg._PoseWithCovarianceStamped import PoseWithCovarianceStamped
from uoa_poc3_msgs.msg import r_battery, r_current_optional
from uoa_poc4_msgs.msg import r_pub_robot_state

robot_id = ""
robot_type = ""
space = ""
amcl_pose = []

def callbackAmclpose(data):
    global amcl_pose
    # Set covariance.
    amcl_pose = data.pose.covariance
    # rospy.loginfo(data)

def publishLocalPath():
    try:
        global robot_id, robot_type, space, amcl_pose
        slash = "/"
        parent_frame = "map"
        child_frame = slash + robot_id + slash + "base_link"
        jst = pytz.timezone('Asia/Tokyo')
        mode = "navi"

        # Robot_status publisher.
        pub_ = rospy.Publisher(robot_id + "/robot_status", r_pub_robot_state, queue_size=10)
        msg = r_pub_robot_state()
        
        tf_listener_ = tf.TransformListener()
        rate = rospy.Rate(1)
        
        while not rospy.is_shutdown():
            try:
                if not tf_listener_.canTransform(parent_frame, child_frame, rospy.Time(0)):
                    # If you cannot get the tf transform.
                    continue
                
                # Wait until tf transform is available
                tf_listener_.waitForTransform(parent_frame, child_frame, rospy.Time(0), rospy.Duration(15.0))
                time_stamp = rospy.Time.now()
                # Get tf transform.
                (trans, rot) = tf_listener_.lookupTransform(parent_frame, child_frame, rospy.Time(0))
                dt = datetime.now(jst)
                iso_time = dt.strftime("%FT%T.%f")[:-3]
                time_zone = dt.strftime("%z")
                dst = iso_time + time_zone
                
                msg.id = robot_id           # Set robot's id.
                msg.type = robot_type       # Set robot's type.
                msg.space = space           # Set space info.
                msg.time = dst              # Set recieved time.
                msg.mode = mode             # Set mode.
                # Set the robot's position.
                msg.pose.position.x = trans[0]
                msg.pose.position.y = trans[1]
                msg.pose.position.z = 0.0
                msg.pose.orientation.x = rot[0]
                msg.pose.orientation.y = rot[1]
                msg.pose.orientation.z = rot[2]
                msg.pose.orientation.w = rot[3]
                # Set covariance.
                msg.covariance = amcl_pose
                # Set battery info.
                msg.battery.voltage = 0.0
                msg.battery.current_optional.valid = False
                msg.battery.current_optional.current = 0.0
                
                pub_.publish(msg)
                
                rospy.loginfo(msg)

            except (tf.LookupException, tf.ConnectivityException, tf.ExtrapolationException):
                continue
            
            finally:
                rate.sleep()
            
    except Exception as err:
        print(err)

def tf_proc():
    global robot_id, robot_type, space
    
    # Get rosparam.
    robot_id = rospy.get_param("id", "robot_id")
    robot_type = rospy.get_param("type", "robot_type")
    space = rospy.get_param("space", "real")
    
    rospy.init_node('tf_proc', anonymous=True)
    
    rospy.loginfo('robot_id: ' + robot_id + ', robot_type: ' + robot_type + ', space: ' + space)
    # Amcl_pose subscriber.
    rospy.Subscriber(robot_id + "/amcl_pose", PoseWithCovarianceStamped, callbackAmclpose)
    rospy.loginfo('tf_proc node started.')
    
    # Set local path.
    publishLocalPath()
    
    rospy.spin()

if __name__ == '__main__':
    try:
        tf_proc()
        
    except rospy.ROSInterruptException:
        pass
    
    finally:
        rospy.loginfo('tf_proc node finished.')
